#sp-progres {
    text-align: center;
    font-weight: bold;
    color: red;
    text-decoration: blink;
}

#sp-options tr th {
    text-align: left;
    font-weight: bold;

}
#sp-options tr td {
    vertical-align: top;
    padding-right: 3em;
}

.sp-footer {
    margin-top: 1em;
    border-top: 1pt solid #aaaaaa;
    font-size: 7pt;
}

#document-action-smartprintng-export {
    font-weight: bold;
}

.sp-option {
    padding-bottom: 0.5em;
}
